package com.hzya.frame.plugin.lets.util;

import cn.hutool.core.lang.Assert;
import com.hzya.frame.plugin.lets.dao.IIaPeriodaccountDao;
import com.hzya.frame.plugin.lets.dao.IPoOrderBDao;
import com.hzya.frame.plugin.lets.entity.BdInvbasdocEntity;
import com.hzya.frame.plugin.lets.entity.IaPeriodaccountEntity;
import com.hzya.frame.plugin.lets.entity.PoOrderBEntity;
import com.hzya.frame.split.SplitListByCountUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 得到存货的结存单价和采购价
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.util
 * @Project：kangarooDataCenterV3
 * @name：BalanceUnitPriceUtil
 * @Date：2024/10/17 09:17
 * @Filename：BalanceUnitPriceUtil
 */
@Component
public class BalanceUnitPriceUtil {

    @Autowired
    private IIaPeriodaccountDao iIaPeriodaccountDao;

    @Autowired
    private IPoOrderBDao iPoOrderBDao;

    private static final Map<String, String> cacheShopMap = new HashMap<>();

    static {
        cacheShopMap.put("FANG-Q-L", "FANG-Q-L");
        cacheShopMap.put("dy-FQL", "dy-FQL");
        cacheShopMap.put("hzz", "hzz");
        cacheShopMap.put("pdd-gunanxi", "pdd-gunanxi");
        cacheShopMap.put("pdd-yanxi", "pdd-yanxi");
    }

    /**
     * 判断是否取结存价的店铺
     *
     * @param ofsShop OFS店铺编码
     * @author liuyang
     */
    public Boolean checkOfsShop(String ofsShop) {
        Assert.notNull(ofsShop, "OFS店铺编码不能为空！");
        String cacheValue = cacheShopMap.get(ofsShop);
        if (cacheValue == null) {
            return false;
        }
        return true;
    }

    /**
     * 查询存货的结存金额、数量等信息
     * 需要手动计算一遍，用金额/数量，可能会有负结存的情况，我取绝对值
     *
     * @author liuyang
     */
    public List<IaPeriodaccountEntity> queryBalanceUnitPrice(List<BdInvbasdocEntity> bdInvbasdocEntityList) throws Exception {
        List<IaPeriodaccountEntity> iaPeriodaccountEntityList = new ArrayList<>();
        if (bdInvbasdocEntityList != null && bdInvbasdocEntityList.size() > 0) {
            List<List<BdInvbasdocEntity>> splitListByCount = SplitListByCountUtil.splitListByCount(bdInvbasdocEntityList, 100);
            for (int i = 0; i < splitListByCount.size(); i++) {
                List<BdInvbasdocEntity> bdInvbasdocEntities = splitListByCount.get(i);

                String invcodes = bdInvbasdocEntities.stream().map(bdInvbasdoc -> "'" + bdInvbasdoc.getInvcode() + "'").collect(Collectors.joining(","));
                IaPeriodaccountEntity iaPeriodaccountEntity = new IaPeriodaccountEntity();
                iaPeriodaccountEntity.setDataSourceCode("lets_u8c");
                iaPeriodaccountEntity.setInvcodes(invcodes);
                List<IaPeriodaccountEntity> iaPeriodaccountEntityList1 = iIaPeriodaccountDao.queryBalancePrice(iaPeriodaccountEntity);
                iaPeriodaccountEntityList.addAll(iaPeriodaccountEntityList1);
            }
        }
        return iaPeriodaccountEntityList;
    }

    /**
     * 查询存货对应的采购单价
     *
     * @author liuyang
     */
    public List<PoOrderBEntity> queryPurchaseUnitPriceByInvcodes(List<BdInvbasdocEntity> bdInvbasdocEntityList) throws Exception {
        List<PoOrderBEntity> poOrderBEntityList = new ArrayList<>();
        if (bdInvbasdocEntityList != null && bdInvbasdocEntityList.size() > 0) {
            List<List<BdInvbasdocEntity>> splitListByCount = SplitListByCountUtil.splitListByCount(bdInvbasdocEntityList, 100);
            for (int i = 0; i < splitListByCount.size(); i++) {
                List<BdInvbasdocEntity> bdInvbasdocEntities = splitListByCount.get(i);
                String invcodes = bdInvbasdocEntities.stream().map(bdInvbasdoc -> "'" + bdInvbasdoc.getInvcode() + "'").collect(Collectors.joining(","));

                PoOrderBEntity poOrderBEntity = new PoOrderBEntity();
                poOrderBEntity.setInvcodes(invcodes);
                poOrderBEntity.setDataSourceCode("lets_u8c");
                List<PoOrderBEntity> poOrderBEntityList1 = iPoOrderBDao.queryPurchaseUnitPriceByInvcodes(poOrderBEntity);
                poOrderBEntityList.addAll(poOrderBEntityList1);
            }
        }
        return poOrderBEntityList;
    }
}